# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .user_role_with_resource_resource_type import UserRoleWithResourceResourceType


class UserRoleWithResource(UniversalBaseModel):
    role_id: typing_extensions.Annotated[str, FieldMetadata(alias="roleId")]
    resource_type: typing_extensions.Annotated[UserRoleWithResourceResourceType, FieldMetadata(alias="resourceType")]
    resource_id: typing_extensions.Annotated[str, FieldMetadata(alias="resourceId")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .artifact_type import ArtifactType
from .base_artifact_version import BaseArtifactVersion
from .subject import Subject


class Artifact(UniversalBaseModel):
    id: str
    ml_repo_id: str
    type: ArtifactType
    name: str
    fqn: str
    created_by_subject: Subject
    created_at: typing.Optional[dt.datetime] = None
    updated_at: typing.Optional[dt.datetime] = None
    latest_version: typing.Optional[BaseArtifactVersion] = None
    run_steps: typing.Optional[typing.List[int]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class NatsMetricConfig(UniversalBaseModel):
    type: typing.Literal["nats"] = pydantic.Field(default="nats")
    """
    +value=nats
    """

    lag_threshold: int = pydantic.Field()
    """
    Upper limit of the number of backlog messages the auto-scaler will try to maintain per replica. If you set this number to 10 and have 30 messages in the stream and one replica, the auto-scaler will scale the number of replicas to 3.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AzureOAuth(UniversalBaseModel):
    """
    Azure OAuth
    """

    type: typing.Literal["oauth"] = pydantic.Field(default="oauth")
    """
    +value=oauth
    """

    tenant_id: str = pydantic.Field()
    """
    The Azure Active Directory tenant (directory) ID.
    """

    client_id: str = pydantic.Field()
    """
    The client (application) ID of an App Registration in the tenant.
    """

    client_secret: str = pydantic.Field()
    """
    A client secret that was generated for the App Registration.
    """

    subscription_id: str = pydantic.Field()
    """
    The ID of the target subscription. The value must be an UUID.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core import enum

T_Result = typing.TypeVar("T_Result")


class ApplicationsListRequestLifecycleStage(enum.StrEnum):
    ACTIVE = "active"
    DELETING = "deleting"
    DELETION_FAILED = "deletion_failed"

    def visit(
        self,
        active: typing.Callable[[], T_Result],
        deleting: typing.Callable[[], T_Result],
        deletion_failed: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ApplicationsListRequestLifecycleStage.ACTIVE:
            return active()
        if self is ApplicationsListRequestLifecycleStage.DELETING:
            return deleting()
        if self is ApplicationsListRequestLifecycleStage.DELETION_FAILED:
            return deletion_failed()



# This file was auto-generated by Fern from our API Definition.

import typing

from .basic_auth_creds import BasicAuthCreds
from .jwt_auth_config import JwtAuthConfig
from .true_foundry_interactive_login import TrueFoundryInteractiveLogin

PortAuth = typing.Union[BasicAuthCreds, JwtAuthConfig, TrueFoundryInteractiveLogin]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .subject import Subject


class TraceSpan(UniversalBaseModel):
    span_id: typing_extensions.Annotated[str, FieldMetadata(alias="spanId")] = pydantic.Field()
    """
    Unique identifier for the span within the trace.
    """

    trace_id: typing_extensions.Annotated[str, FieldMetadata(alias="traceId")] = pydantic.Field()
    """
    Unique identifier for the trace that contains this span.
    """

    parent_span_id: typing_extensions.Annotated[str, FieldMetadata(alias="parentSpanId")] = pydantic.Field()
    """
    Identifier of the parent span in the trace hierarchy.
    """

    service_name: typing_extensions.Annotated[str, FieldMetadata(alias="serviceName")] = pydantic.Field()
    """
    Name of the service that generated this span.
    """

    span_name: typing_extensions.Annotated[str, FieldMetadata(alias="spanName")] = pydantic.Field()
    """
    Name of the span
    """

    span_kind: typing_extensions.Annotated[str, FieldMetadata(alias="spanKind")] = pydantic.Field()
    """
    Type of span (e.g., CLIENT, SERVER, INTERNAL, PRODUCER, CONSUMER).
    """

    scope_name: typing_extensions.Annotated[str, FieldMetadata(alias="scopeName")] = pydantic.Field()
    """
    Name of the instrumentation scope that created this span.
    """

    scope_version: typing_extensions.Annotated[str, FieldMetadata(alias="scopeVersion")] = pydantic.Field()
    """
    Version of the instrumentation scope that created this span.
    """

    timestamp: str = pydantic.Field()
    """
    Timestamp in ISO 8601 format (e.g., 2025-03-12T00:00:09.872Z).
    """

    duration_ns: typing_extensions.Annotated[float, FieldMetadata(alias="durationNs")] = pydantic.Field()
    """
    Duration of the span in nanoseconds.
    """

    status_code: typing_extensions.Annotated[str, FieldMetadata(alias="statusCode")] = pydantic.Field()
    """
    Status code of the span (e.g., OK, ERROR, UNSET).
    """

    status_message: typing_extensions.Annotated[str, FieldMetadata(alias="statusMessage")] = pydantic.Field()
    """
    Human-readable status message describing the span result.
    """

    span_attributes: typing_extensions.Annotated[
        typing.Dict[str, typing.Any], FieldMetadata(alias="spanAttributes")
    ] = pydantic.Field()
    """
    Key-value pairs containing additional metadata about the span.
    """

    events: typing.List[typing.Dict[str, typing.Any]] = pydantic.Field()
    """
    Array of events that occurred during the span execution.
    """

    created_by_subject: typing_extensions.Annotated[Subject, FieldMetadata(alias="createdBySubject")] = pydantic.Field()
    """
    Subject (user or virtualaccount) that created this span.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .helm_source import HelmSource
from .kustomize import Kustomize


class Helm(UniversalBaseModel):
    type: typing.Literal["helm"] = pydantic.Field(default="helm")
    """
    +value=helm
    """

    name: str = pydantic.Field()
    """
    Name of the Helm deployment. This will be set as the release name of the chart you are deploying.
    """

    labels: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Add labels to base argo app
    """

    source: HelmSource = pydantic.Field()
    """
    Source helm repository
    """

    values: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    Values file as block file
    """

    kustomize: typing.Optional[Kustomize] = None
    ignore_differences: typing_extensions.Annotated[
        typing.Optional[typing.List[typing.Dict[str, typing.Any]]], FieldMetadata(alias="ignoreDifferences")
    ] = None
    workspace_fqn: typing.Optional[str] = pydantic.Field(default=None)
    """
    Fully qualified name of the workspace
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SmtpCredentials(UniversalBaseModel):
    """
    SMTP Credentials
    """

    type: typing.Literal["smtp-credentials"] = pydantic.Field(default="smtp-credentials")
    """
    +value=smtp-credentials
    +sort=250
    """

    host: str = pydantic.Field()
    """
    The SMTP host of the email server.
    """

    username: str = pydantic.Field()
    """
    The username for the SMTP server.
    """

    password: str = pydantic.Field()
    """
    The password for the SMTP server.
    """

    port: int = pydantic.Field(default=587)
    """
    The port of the SMTP server.
    """

    tls: bool = pydantic.Field(default=True)
    """
    Whether to use TLS for the SMTP server.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .rate_limit_rule import RateLimitRule


class RateLimitConfig(UniversalBaseModel):
    """
    Rate Limit Configuration
    """

    name: str = pydantic.Field()
    """
    Name of the rate limiting configuration
    """

    type: typing.Literal["gateway-rate-limiting-config"] = pydantic.Field(default="gateway-rate-limiting-config")
    """
    +value=gateway-rate-limiting-config
    +sort=2
    """

    rules: typing.List[RateLimitRule] = pydantic.Field()
    """
    List of rate limit rules
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .json_schema import JsonSchema


class JsonSchemaResponseFormat(UniversalBaseModel):
    """
    Structured JSON response format with schema validation
    """

    type: typing.Literal["json_schema"] = pydantic.Field(default="json_schema")
    """
    Type of response format
    """

    json_schema: JsonSchema

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class TaskDockerFileBuild(UniversalBaseModel):
    """
    Describes the configuration for the docker build for a task
    """

    type: typing.Literal["task-dockerfile-build"] = pydantic.Field(default="task-dockerfile-build")
    """
    +value=dockerfile
    """

    docker_registry: typing.Optional[str] = pydantic.Field(default=None)
    """
    FQN of the container registry. If you can't find your registry here,
    add it through the [Integrations](/integrations?tab=docker-registry) page
    """

    dockerfile_path: str = pydantic.Field(default="./Dockerfile")
    """
    The file path of the Dockerfile relative to project root path.
    """

    build_args: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Build arguments to pass to docker build
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class FunctionSchema(UniversalBaseModel):
    """
    Schema defining a function for tool calls
    """

    name: str = pydantic.Field()
    """
    Name of the function
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the function
    """

    parameters: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    Parameters schema for the function
    """

    strict: typing.Optional[bool] = pydantic.Field(default=False)
    """
    Indicates if the function should be called strictly
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ClouderaWorkloadAuthToken(UniversalBaseModel):
    """
    Authenticate using Cloudera CDP workload authentication (recommended). This method generates a secure workload token using your API access key and private key, allowing you to securely access CDP workloads and services.
    """

    type: typing.Literal["workload-auth-token"] = pydantic.Field(default="workload-auth-token")
    """
    +value=workload-auth-token
    """

    cdp_base_url: str = pydantic.Field()
    """
    The base URL for CDP API services
    """

    access_key_id: str = pydantic.Field()
    """
    The access key ID for your Cloudera CDP workload authentication.
    """

    private_key: str = pydantic.Field()
    """
    The private key for your Cloudera CDP workload authentication.
    """

    workload_name: str = pydantic.Field()
    """
    The name of the CDP workload you want to authenticate with.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..types.get_logs_response import GetLogsResponse
from ..types.logs_search_filter_type import LogsSearchFilterType
from ..types.logs_search_operator_type import LogsSearchOperatorType
from ..types.logs_sorting_direction import LogsSortingDirection


class RawLogsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        *,
        start_ts: typing.Optional[int] = None,
        end_ts: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        direction: typing.Optional[LogsSortingDirection] = None,
        num_logs_to_ignore: typing.Optional[int] = None,
        application_id: typing.Optional[str] = None,
        application_fqn: typing.Optional[str] = None,
        deployment_id: typing.Optional[str] = None,
        job_run_name: typing.Optional[str] = None,
        pod_name: typing.Optional[str] = None,
        container_name: typing.Optional[str] = None,
        pod_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        pod_names_regex: typing.Optional[str] = None,
        search_filters: typing.Optional[str] = None,
        search_string: typing.Optional[str] = None,
        search_type: typing.Optional[LogsSearchFilterType] = None,
        search_operator: typing.Optional[LogsSearchOperatorType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetLogsResponse]:
        """
        Fetch logs for various workload components, including Services, Jobs, Workflows, Job Runs, and Pods. Logs are filtered based on the provided query parameters.

        Parameters
        ----------
        start_ts : typing.Optional[int]
            Start timestamp for querying logs, in nanoseconds from the Unix epoch.

        end_ts : typing.Optional[int]
            End timestamp for querying logs, in nanoseconds from the Unix epoch.

        limit : typing.Optional[int]
            Max number of log lines to fetch

        direction : typing.Optional[LogsSortingDirection]
            Direction of sorting logs. Can be `asc` or `desc`

        num_logs_to_ignore : typing.Optional[int]
            Number of logs corresponding to the starting timestamp to be ignored.

        application_id : typing.Optional[str]
            Application ID

        application_fqn : typing.Optional[str]
            Application FQN

        deployment_id : typing.Optional[str]
            Deployment ID

        job_run_name : typing.Optional[str]
            Name of the Job Run for which to fetch logs.

        pod_name : typing.Optional[str]
            Name of Pod for which to fetch logs.

        container_name : typing.Optional[str]
            Name of the Container for which to fetch logs.

        pod_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            List of pod names for which to fetch logs.

        pod_names_regex : typing.Optional[str]
            Regex pattern for pod names to fetch logs.

        search_filters : typing.Optional[str]
            JSON string containing array of search filters with string, type and operator

        search_string : typing.Optional[str]
            String that needs to be matched

        search_type : typing.Optional[LogsSearchFilterType]
            Query filter type, `regex` `substring` `ignore_case_substring`

        search_operator : typing.Optional[LogsSearchOperatorType]
            Comparison operator for filter. `equal` or `not_equal`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetLogsResponse]
            Successfully retrieved logs for the workload
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/svc/v1/logs",
            method="GET",
            params={
                "startTs": start_ts,
                "endTs": end_ts,
                "limit": limit,
                "direction": direction,
                "numLogsToIgnore": num_logs_to_ignore,
                "applicationId": application_id,
                "applicationFqn": application_fqn,
                "deploymentId": deployment_id,
                "jobRunName": job_run_name,
                "podName": pod_name,
                "containerName": container_name,
                "podNames": pod_names,
                "podNamesRegex": pod_names_regex,
                "searchFilters": search_filters,
                "searchString": search_string,
                "searchType": search_type,
                "searchOperator": search_operator,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLogsResponse,
                    parse_obj_as(
                        type_=GetLogsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLogsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        *,
        start_ts: typing.Optional[int] = None,
        end_ts: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        direction: typing.Optional[LogsSortingDirection] = None,
        num_logs_to_ignore: typing.Optional[int] = None,
        application_id: typing.Optional[str] = None,
        application_fqn: typing.Optional[str] = None,
        deployment_id: typing.Optional[str] = None,
        job_run_name: typing.Optional[str] = None,
        pod_name: typing.Optional[str] = None,
        container_name: typing.Optional[str] = None,
        pod_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        pod_names_regex: typing.Optional[str] = None,
        search_filters: typing.Optional[str] = None,
        search_string: typing.Optional[str] = None,
        search_type: typing.Optional[LogsSearchFilterType] = None,
        search_operator: typing.Optional[LogsSearchOperatorType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetLogsResponse]:
        """
        Fetch logs for various workload components, including Services, Jobs, Workflows, Job Runs, and Pods. Logs are filtered based on the provided query parameters.

        Parameters
        ----------
        start_ts : typing.Optional[int]
            Start timestamp for querying logs, in nanoseconds from the Unix epoch.

        end_ts : typing.Optional[int]
            End timestamp for querying logs, in nanoseconds from the Unix epoch.

        limit : typing.Optional[int]
            Max number of log lines to fetch

        direction : typing.Optional[LogsSortingDirection]
            Direction of sorting logs. Can be `asc` or `desc`

        num_logs_to_ignore : typing.Optional[int]
            Number of logs corresponding to the starting timestamp to be ignored.

        application_id : typing.Optional[str]
            Application ID

        application_fqn : typing.Optional[str]
            Application FQN

        deployment_id : typing.Optional[str]
            Deployment ID

        job_run_name : typing.Optional[str]
            Name of the Job Run for which to fetch logs.

        pod_name : typing.Optional[str]
            Name of Pod for which to fetch logs.

        container_name : typing.Optional[str]
            Name of the Container for which to fetch logs.

        pod_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            List of pod names for which to fetch logs.

        pod_names_regex : typing.Optional[str]
            Regex pattern for pod names to fetch logs.

        search_filters : typing.Optional[str]
            JSON string containing array of search filters with string, type and operator

        search_string : typing.Optional[str]
            String that needs to be matched

        search_type : typing.Optional[LogsSearchFilterType]
            Query filter type, `regex` `substring` `ignore_case_substring`

        search_operator : typing.Optional[LogsSearchOperatorType]
            Comparison operator for filter. `equal` or `not_equal`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetLogsResponse]
            Successfully retrieved logs for the workload
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/svc/v1/logs",
            method="GET",
            params={
                "startTs": start_ts,
                "endTs": end_ts,
                "limit": limit,
                "direction": direction,
                "numLogsToIgnore": num_logs_to_ignore,
                "applicationId": application_id,
                "applicationFqn": application_fqn,
                "deploymentId": deployment_id,
                "jobRunName": job_run_name,
                "podName": pod_name,
                "containerName": container_name,
                "podNames": pod_names,
                "podNamesRegex": pod_names_regex,
                "searchFilters": search_filters,
                "searchString": search_string,
                "searchType": search_type,
                "searchOperator": search_operator,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLogsResponse,
                    parse_obj_as(
                        type_=GetLogsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .model_cost_metric import ModelCostMetric
from .model_type import ModelType


class SambaNovaModel(UniversalBaseModel):
    """
    SambaNova Model
    """

    type: typing.Literal["integration/model/sambanova"] = pydantic.Field(default="integration/model/sambanova")
    """
    +value=integration/model/sambanova
    """

    name: str = pydantic.Field()
    """
    A descriptive name to identify this model integration in the UI
    """

    model_id: str = pydantic.Field()
    """
    The SambaNova model identifier (e.g., DeepSeek-R1-0528, Meta-Llama-3.3-70B-Instruct). This is the standard Model ID from SambaNova's model catalog.
    """

    model_types: typing.List[ModelType] = pydantic.Field()
    """
    Specify the type of the SambaNova model
    """

    cost: typing.Optional[ModelCostMetric] = None
    authorized_subjects: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of subjects that are authorized to access this integration. List of user fqn in format <user_type>:<username>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .aws_region import AwsRegion
from .aws_sagemaker_provider_account_auth_data import AwsSagemakerProviderAccountAuthData
from .collaborator import Collaborator
from .owned_by import OwnedBy
from .sagemaker_model import SagemakerModel


class AwsSagemakerProviderAccount(UniversalBaseModel):
    """
    Sagemaker
    """

    type: typing.Literal["provider-account/aws-sagemaker"] = pydantic.Field(default="provider-account/aws-sagemaker")
    """
    +value=provider-account/aws-sagemaker
    """

    name: str = pydantic.Field()
    """
    The name of the Sagemaker provider account
    """

    region: AwsRegion
    auth_data: typing.Optional[AwsSagemakerProviderAccountAuthData] = pydantic.Field(default=None)
    """
    Authentication data for the AWS account
    """

    integrations: typing.List[SagemakerModel] = pydantic.Field()
    """
    List of integrations that are associated with the Sagemaker provider account
    """

    collaborators: typing.Optional[typing.List[Collaborator]] = pydantic.Field(default=None)
    """
    List of users who have access to this provider account
    """

    owned_by: typing_extensions.Annotated[typing.Optional[OwnedBy], FieldMetadata(alias="ownedBy")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ....core.serialization import FieldMetadata


class DockerRegistriesGetCredentialsResponse(UniversalBaseModel):
    fqn: typing.Optional[str] = pydantic.Field(default=None)
    """
    Docker registry FQN
    """

    registry_url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="registryUrl")] = (
        pydantic.Field(default=None)
    )
    """
    Docker registry URL
    """

    username: typing.Optional[str] = pydantic.Field(default=None)
    """
    Username
    """

    password: typing.Optional[str] = pydantic.Field(default=None)
    """
    Password
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class SubjectType(enum.StrEnum):
    """
    Subject type
    """

    USER = "user"
    TEAM = "team"
    SERVICEACCOUNT = "serviceaccount"
    VIRTUALACCOUNT = "virtualaccount"
    EXTERNAL_IDENTITY = "external-identity"

    def visit(
        self,
        user: typing.Callable[[], T_Result],
        team: typing.Callable[[], T_Result],
        serviceaccount: typing.Callable[[], T_Result],
        virtualaccount: typing.Callable[[], T_Result],
        external_identity: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is SubjectType.USER:
            return user()
        if self is SubjectType.TEAM:
            return team()
        if self is SubjectType.SERVICEACCOUNT:
            return serviceaccount()
        if self is SubjectType.VIRTUALACCOUNT:
            return virtualaccount()
        if self is SubjectType.EXTERNAL_IDENTITY:
            return external_identity()



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class InferMethodName(enum.StrEnum):
    """
    Name of the method used for inference
    """

    PREDICT = "predict"
    PREDICT_PROBA = "predict_proba"

    def visit(self, predict: typing.Callable[[], T_Result], predict_proba: typing.Callable[[], T_Result]) -> T_Result:
        if self is InferMethodName.PREDICT:
            return predict()
        if self is InferMethodName.PREDICT_PROBA:
            return predict_proba()



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class DeploymentStatusValue(enum.StrEnum):
    INITIALIZED = "INITIALIZED"
    BUILD_SUCCESS = "BUILD_SUCCESS"
    BUILD_FAILED = "BUILD_FAILED"
    DEPLOY_FAILED = "DEPLOY_FAILED"
    DEPLOY_FAILED_WITH_RETRY = "DEPLOY_FAILED_WITH_RETRY"
    DEPLOY_SUCCESS = "DEPLOY_SUCCESS"
    ROLLOUT_STARTED = "ROLLOUT_STARTED"
    SET_TRAFFIC = "SET_TRAFFIC"
    PAUSED = "PAUSED"
    FAILED = "FAILED"
    CANCELLED = "CANCELLED"
    REDEPLOY_STARTED = "REDEPLOY_STARTED"
    BUILDING = "BUILDING"

    def visit(
        self,
        initialized: typing.Callable[[], T_Result],
        build_success: typing.Callable[[], T_Result],
        build_failed: typing.Callable[[], T_Result],
        deploy_failed: typing.Callable[[], T_Result],
        deploy_failed_with_retry: typing.Callable[[], T_Result],
        deploy_success: typing.Callable[[], T_Result],
        rollout_started: typing.Callable[[], T_Result],
        set_traffic: typing.Callable[[], T_Result],
        paused: typing.Callable[[], T_Result],
        failed: typing.Callable[[], T_Result],
        cancelled: typing.Callable[[], T_Result],
        redeploy_started: typing.Callable[[], T_Result],
        building: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is DeploymentStatusValue.INITIALIZED:
            return initialized()
        if self is DeploymentStatusValue.BUILD_SUCCESS:
            return build_success()
        if self is DeploymentStatusValue.BUILD_FAILED:
            return build_failed()
        if self is DeploymentStatusValue.DEPLOY_FAILED:
            return deploy_failed()
        if self is DeploymentStatusValue.DEPLOY_FAILED_WITH_RETRY:
            return deploy_failed_with_retry()
        if self is DeploymentStatusValue.DEPLOY_SUCCESS:
            return deploy_success()
        if self is DeploymentStatusValue.ROLLOUT_STARTED:
            return rollout_started()
        if self is DeploymentStatusValue.SET_TRAFFIC:
            return set_traffic()
        if self is DeploymentStatusValue.PAUSED:
            return paused()
        if self is DeploymentStatusValue.FAILED:
            return failed()
        if self is DeploymentStatusValue.CANCELLED:
            return cancelled()
        if self is DeploymentStatusValue.REDEPLOY_STARTED:
            return redeploy_started()
        if self is DeploymentStatusValue.BUILDING:
            return building()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class VolumeMount(UniversalBaseModel):
    type: typing.Literal["volume"] = pydantic.Field(default="volume")
    """
    +value=volume
    """

    mount_path: str = pydantic.Field()
    """
    Absolute file path where the volume will be mounted.
    """

    sub_path: typing.Optional[str] = pydantic.Field(default=None)
    """
    Sub path within the volume to mount. Defaults to root of the volume.
    """

    volume_fqn: str = pydantic.Field()
    """
    The TrueFoundry volume that needs to be mounted.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .span_field_filter_operator import SpanFieldFilterOperator
from .span_field_filter_span_field_name import SpanFieldFilterSpanFieldName
from .span_field_filter_value import SpanFieldFilterValue


class SpanFieldFilter(UniversalBaseModel):
    span_field_name: typing_extensions.Annotated[SpanFieldFilterSpanFieldName, FieldMetadata(alias="spanFieldName")]
    operator: SpanFieldFilterOperator
    value: SpanFieldFilterValue

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class TrueFoundryApplyResponseAction(enum.StrEnum):
    """
    The action performed: CREATE or UPDATE
    """

    CREATE = "CREATE"
    UPDATE = "UPDATE"

    def visit(self, create: typing.Callable[[], T_Result], update: typing.Callable[[], T_Result]) -> T_Result:
        if self is TrueFoundryApplyResponseAction.CREATE:
            return create()
        if self is TrueFoundryApplyResponseAction.UPDATE:
            return update()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AwsAccessKeyAuth(UniversalBaseModel):
    aws_access_key_id: str = pydantic.Field()
    """
    AWS Access Key ID
    """

    aws_secret_access_key: str = pydantic.Field()
    """
    AWS Secret Access Key for the user to authenticate with
    """

    aws_session_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS Session Token, only required when using temporary credentials
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.sso_team_manifest import SsoTeamManifest
from ...types.team_manifest import TeamManifest

ApplyTeamRequestManifest = typing.Union[TeamManifest, SsoTeamManifest]



# This file was auto-generated by Fern from our API Definition.

import typing

from .aws_sqs_access_key_based_auth import AwsSqsAccessKeyBasedAuth
from .aws_sqs_assumed_role_based_auth import AwsSqsAssumedRoleBasedAuth

SqsQueueMetricConfigAuth = typing.Union[AwsSqsAccessKeyBasedAuth, AwsSqsAssumedRoleBasedAuth]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.internal_list_artifact_versions_response import InternalListArtifactVersionsResponse
from ...types.internal_list_artifact_versions_response_data_item import InternalListArtifactVersionsResponseDataItem
from .raw_client import AsyncRawArtifactVersionsClient, RawArtifactVersionsClient


class ArtifactVersionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawArtifactVersionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawArtifactVersionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawArtifactVersionsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        tag: typing.Optional[str] = None,
        fqn: typing.Optional[str] = None,
        artifact_id: typing.Optional[str] = None,
        ml_repo_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        version: typing.Optional[int] = None,
        run_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        run_steps: typing.Optional[typing.Union[int, typing.Sequence[int]]] = None,
        offset: typing.Optional[int] = 0,
        limit: typing.Optional[int] = 100,
        include_internal_metadata: typing.Optional[bool] = False,
        include_model_versions: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[InternalListArtifactVersionsResponseDataItem, InternalListArtifactVersionsResponse]:
        """
        List artifact version API

        Parameters
        ----------
        tag : typing.Optional[str]

        fqn : typing.Optional[str]

        artifact_id : typing.Optional[str]

        ml_repo_id : typing.Optional[str]

        name : typing.Optional[str]

        version : typing.Optional[int]

        run_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        run_steps : typing.Optional[typing.Union[int, typing.Sequence[int]]]

        offset : typing.Optional[int]

        limit : typing.Optional[int]

        include_internal_metadata : typing.Optional[bool]

        include_model_versions : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[InternalListArtifactVersionsResponseDataItem, InternalListArtifactVersionsResponse]
            Successful Response

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        response = client.internal.artifact_versions.list(
            tag="tag",
            fqn="fqn",
            artifact_id="artifact_id",
            ml_repo_id="ml_repo_id",
            name="name",
            version=1,
            offset=1,
            limit=1,
            include_internal_metadata=True,
            include_model_versions=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            tag=tag,
            fqn=fqn,
            artifact_id=artifact_id,
            ml_repo_id=ml_repo_id,
            name=name,
            version=version,
            run_ids=run_ids,
            run_steps=run_steps,
            offset=offset,
            limit=limit,
            include_internal_metadata=include_internal_metadata,
            include_model_versions=include_model_versions,
            request_options=request_options,
        )


class AsyncArtifactVersionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawArtifactVersionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawArtifactVersionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawArtifactVersionsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        tag: typing.Optional[str] = None,
        fqn: typing.Optional[str] = None,
        artifact_id: typing.Optional[str] = None,
        ml_repo_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        version: typing.Optional[int] = None,
        run_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        run_steps: typing.Optional[typing.Union[int, typing.Sequence[int]]] = None,
        offset: typing.Optional[int] = 0,
        limit: typing.Optional[int] = 100,
        include_internal_metadata: typing.Optional[bool] = False,
        include_model_versions: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[InternalListArtifactVersionsResponseDataItem, InternalListArtifactVersionsResponse]:
        """
        List artifact version API

        Parameters
        ----------
        tag : typing.Optional[str]

        fqn : typing.Optional[str]

        artifact_id : typing.Optional[str]

        ml_repo_id : typing.Optional[str]

        name : typing.Optional[str]

        version : typing.Optional[int]

        run_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        run_steps : typing.Optional[typing.Union[int, typing.Sequence[int]]]

        offset : typing.Optional[int]

        limit : typing.Optional[int]

        include_internal_metadata : typing.Optional[bool]

        include_model_versions : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[InternalListArtifactVersionsResponseDataItem, InternalListArtifactVersionsResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            response = await client.internal.artifact_versions.list(
                tag="tag",
                fqn="fqn",
                artifact_id="artifact_id",
                ml_repo_id="ml_repo_id",
                name="name",
                version=1,
                offset=1,
                limit=1,
                include_internal_metadata=True,
                include_model_versions=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            tag=tag,
            fqn=fqn,
            artifact_id=artifact_id,
            ml_repo_id=ml_repo_id,
            name=name,
            version=version,
            run_ids=run_ids,
            run_steps=run_steps,
            offset=offset,
            limit=limit,
            include_internal_metadata=include_internal_metadata,
            include_model_versions=include_model_versions,
            request_options=request_options,
        )



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class AzurePiiCategory(enum.StrEnum):
    ALL = "all"
    PERSON = "Person"
    PERSON_TYPE = "PersonType"
    PHONE_NUMBER = "PhoneNumber"
    ORGANIZATION = "Organization"
    ADDRESS = "Address"
    EMAIL = "Email"
    URL = "URL"
    IP_ADDRESS = "IP Address"
    DATE_TIME = "DateTime"
    BANK_ACCOUNT_NUMBER = "BankAccountNumber"
    DRIVERS_LICENSE_NUMBER = "DriversLicenseNumber"
    PASSPORT_NUMBER = "PassportNumber"

    def visit(
        self,
        all_: typing.Callable[[], T_Result],
        person: typing.Callable[[], T_Result],
        person_type: typing.Callable[[], T_Result],
        phone_number: typing.Callable[[], T_Result],
        organization: typing.Callable[[], T_Result],
        address: typing.Callable[[], T_Result],
        email: typing.Callable[[], T_Result],
        url: typing.Callable[[], T_Result],
        ip_address: typing.Callable[[], T_Result],
        date_time: typing.Callable[[], T_Result],
        bank_account_number: typing.Callable[[], T_Result],
        drivers_license_number: typing.Callable[[], T_Result],
        passport_number: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is AzurePiiCategory.ALL:
            return all_()
        if self is AzurePiiCategory.PERSON:
            return person()
        if self is AzurePiiCategory.PERSON_TYPE:
            return person_type()
        if self is AzurePiiCategory.PHONE_NUMBER:
            return phone_number()
        if self is AzurePiiCategory.ORGANIZATION:
            return organization()
        if self is AzurePiiCategory.ADDRESS:
            return address()
        if self is AzurePiiCategory.EMAIL:
            return email()
        if self is AzurePiiCategory.URL:
            return url()
        if self is AzurePiiCategory.IP_ADDRESS:
            return ip_address()
        if self is AzurePiiCategory.DATE_TIME:
            return date_time()
        if self is AzurePiiCategory.BANK_ACCOUNT_NUMBER:
            return bank_account_number()
        if self is AzurePiiCategory.DRIVERS_LICENSE_NUMBER:
            return drivers_license_number()
        if self is AzurePiiCategory.PASSPORT_NUMBER:
            return passport_number()



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.empty_response import EmptyResponse
from ..types.get_model_version_response import GetModelVersionResponse
from ..types.list_model_versions_response import ListModelVersionsResponse
from ..types.model_version import ModelVersion
from .raw_client import AsyncRawModelVersionsClient, RawModelVersionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ModelVersionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawModelVersionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawModelVersionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawModelVersionsClient
        """
        return self._raw_client

    def apply_tags(
        self,
        *,
        model_version_id: str,
        tags: typing.Sequence[str],
        force: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmptyResponse:
        """
        Parameters
        ----------
        model_version_id : str

        tags : typing.Sequence[str]

        force : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmptyResponse
            Successful Response

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.model_versions.apply_tags(
            model_version_id="model_version_id",
            tags=["tags"],
        )
        """
        _response = self._raw_client.apply_tags(
            model_version_id=model_version_id, tags=tags, force=force, request_options=request_options
        )
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetModelVersionResponse:
        """
        Get model version API

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetModelVersionResponse
            Successful Response

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.model_versions.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EmptyResponse:
        """
        Delete model versions API

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmptyResponse
            Successful Response

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.model_versions.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def list(
        self,
        *,
        tag: typing.Optional[str] = None,
        fqn: typing.Optional[str] = None,
        model_id: typing.Optional[str] = None,
        ml_repo_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        version: typing.Optional[int] = None,
        run_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        run_steps: typing.Optional[typing.Union[int, typing.Sequence[int]]] = None,
        offset: typing.Optional[int] = 0,
        limit: typing.Optional[int] = 100,
        include_internal_metadata: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ModelVersion, ListModelVersionsResponse]:
        """
        List model version API

        Parameters
        ----------
        tag : typing.Optional[str]

        fqn : typing.Optional[str]

        model_id : typing.Optional[str]

        ml_repo_id : typing.Optional[str]

        name : typing.Optional[str]

        version : typing.Optional[int]

        run_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        run_steps : typing.Optional[typing.Union[int, typing.Sequence[int]]]

        offset : typing.Optional[int]

        limit : typing.Optional[int]

        include_internal_metadata : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ModelVersion, ListModelVersionsResponse]
            Successful Response

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        response = client.model_versions.list(
            tag="tag",
            fqn="fqn",
            model_id="model_id",
            ml_repo_id="ml_repo_id",
            name="name",
            version=1,
            offset=1,
            limit=1,
            include_internal_metadata=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            tag=tag,
            fqn=fqn,
            model_id=model_id,
            ml_repo_id=ml_repo_id,
            name=name,
            version=version,
            run_ids=run_ids,
            run_steps=run_steps,
            offset=offset,
            limit=limit,
            include_internal_metadata=include_internal_metadata,
            request_options=request_options,
        )


class AsyncModelVersionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawModelVersionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawModelVersionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawModelVersionsClient
        """
        return self._raw_client

    async def apply_tags(
        self,
        *,
        model_version_id: str,
        tags: typing.Sequence[str],
        force: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmptyResponse:
        """
        Parameters
        ----------
        model_version_id : str

        tags : typing.Sequence[str]

        force : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmptyResponse
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.model_versions.apply_tags(
                model_version_id="model_version_id",
                tags=["tags"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.apply_tags(
            model_version_id=model_version_id, tags=tags, force=force, request_options=request_options
        )
        return _response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetModelVersionResponse:
        """
        Get model version API

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetModelVersionResponse
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.model_versions.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EmptyResponse:
        """
        Delete model versions API

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmptyResponse
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.model_versions.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def list(
        self,
        *,
        tag: typing.Optional[str] = None,
        fqn: typing.Optional[str] = None,
        model_id: typing.Optional[str] = None,
        ml_repo_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        version: typing.Optional[int] = None,
        run_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        run_steps: typing.Optional[typing.Union[int, typing.Sequence[int]]] = None,
        offset: typing.Optional[int] = 0,
        limit: typing.Optional[int] = 100,
        include_internal_metadata: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ModelVersion, ListModelVersionsResponse]:
        """
        List model version API

        Parameters
        ----------
        tag : typing.Optional[str]

        fqn : typing.Optional[str]

        model_id : typing.Optional[str]

        ml_repo_id : typing.Optional[str]

        name : typing.Optional[str]

        version : typing.Optional[int]

        run_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        run_steps : typing.Optional[typing.Union[int, typing.Sequence[int]]]

        offset : typing.Optional[int]

        limit : typing.Optional[int]

        include_internal_metadata : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ModelVersion, ListModelVersionsResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            response = await client.model_versions.list(
                tag="tag",
                fqn="fqn",
                model_id="model_id",
                ml_repo_id="ml_repo_id",
                name="name",
                version=1,
                offset=1,
                limit=1,
                include_internal_metadata=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            tag=tag,
            fqn=fqn,
            model_id=model_id,
            ml_repo_id=ml_repo_id,
            name=name,
            version=version,
            run_ids=run_ids,
            run_steps=run_steps,
            offset=offset,
            limit=limit,
            include_internal_metadata=include_internal_metadata,
            request_options=request_options,
        )



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class NodeSelectorCapacityType(enum.StrEnum):
    """
    Configure what type of nodes to run the app. By default no placement logic is applied.
    "spot_fallback_on_demand" will try to place the application on spot nodes but will fallback to on-demand when spot nodes are not available.
    "spot" will strictly place the application on spot nodes.
    "on_demand" will strictly place the application on on-demand nodes.
    """

    SPOT_FALLBACK_ON_DEMAND = "spot_fallback_on_demand"
    SPOT = "spot"
    ON_DEMAND = "on_demand"

    def visit(
        self,
        spot_fallback_on_demand: typing.Callable[[], T_Result],
        spot: typing.Callable[[], T_Result],
        on_demand: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is NodeSelectorCapacityType.SPOT_FALLBACK_ON_DEMAND:
            return spot_fallback_on_demand()
        if self is NodeSelectorCapacityType.SPOT:
            return spot()
        if self is NodeSelectorCapacityType.ON_DEMAND:
            return on_demand()



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class AwsRegion(enum.StrEnum):
    AF_SOUTH1 = "af-south-1"
    AP_EAST1 = "ap-east-1"
    AP_NORTHEAST1 = "ap-northeast-1"
    AP_NORTHEAST2 = "ap-northeast-2"
    AP_NORTHEAST3 = "ap-northeast-3"
    AP_SOUTH1 = "ap-south-1"
    AP_SOUTH2 = "ap-south-2"
    AP_SOUTHEAST1 = "ap-southeast-1"
    AP_SOUTHEAST2 = "ap-southeast-2"
    AP_SOUTHEAST3 = "ap-southeast-3"
    AP_SOUTHEAST4 = "ap-southeast-4"
    AP_SOUTHEAST5 = "ap-southeast-5"
    AP_SOUTHEAST7 = "ap-southeast-7"
    CA_CENTRAL1 = "ca-central-1"
    CA_WEST1 = "ca-west-1"
    CN_NORTH1 = "cn-north-1"
    CN_NORTHWEST1 = "cn-northwest-1"
    EU_CENTRAL1 = "eu-central-1"
    EU_CENTRAL2 = "eu-central-2"
    EU_NORTH1 = "eu-north-1"
    EU_SOUTH1 = "eu-south-1"
    EU_SOUTH2 = "eu-south-2"
    EU_WEST1 = "eu-west-1"
    EU_WEST2 = "eu-west-2"
    EU_WEST3 = "eu-west-3"
    IL_CENTRAL1 = "il-central-1"
    ME_CENTRAL1 = "me-central-1"
    ME_SOUTH1 = "me-south-1"
    MX_CENTRAL1 = "mx-central-1"
    SA_EAST1 = "sa-east-1"
    US_EAST1 = "us-east-1"
    US_EAST2 = "us-east-2"
    US_GOV_EAST1 = "us-gov-east-1"
    US_GOV_WEST1 = "us-gov-west-1"
    US_WEST1 = "us-west-1"
    US_WEST2 = "us-west-2"

    def visit(
        self,
        af_south1: typing.Callable[[], T_Result],
        ap_east1: typing.Callable[[], T_Result],
        ap_northeast1: typing.Callable[[], T_Result],
        ap_northeast2: typing.Callable[[], T_Result],
        ap_northeast3: typing.Callable[[], T_Result],
        ap_south1: typing.Callable[[], T_Result],
        ap_south2: typing.Callable[[], T_Result],
        ap_southeast1: typing.Callable[[], T_Result],
        ap_southeast2: typing.Callable[[], T_Result],
        ap_southeast3: typing.Callable[[], T_Result],
        ap_southeast4: typing.Callable[[], T_Result],
        ap_southeast5: typing.Callable[[], T_Result],
        ap_southeast7: typing.Callable[[], T_Result],
        ca_central1: typing.Callable[[], T_Result],
        ca_west1: typing.Callable[[], T_Result],
        cn_north1: typing.Callable[[], T_Result],
        cn_northwest1: typing.Callable[[], T_Result],
        eu_central1: typing.Callable[[], T_Result],
        eu_central2: typing.Callable[[], T_Result],
        eu_north1: typing.Callable[[], T_Result],
        eu_south1: typing.Callable[[], T_Result],
        eu_south2: typing.Callable[[], T_Result],
        eu_west1: typing.Callable[[], T_Result],
        eu_west2: typing.Callable[[], T_Result],
        eu_west3: typing.Callable[[], T_Result],
        il_central1: typing.Callable[[], T_Result],
        me_central1: typing.Callable[[], T_Result],
        me_south1: typing.Callable[[], T_Result],
        mx_central1: typing.Callable[[], T_Result],
        sa_east1: typing.Callable[[], T_Result],
        us_east1: typing.Callable[[], T_Result],
        us_east2: typing.Callable[[], T_Result],
        us_gov_east1: typing.Callable[[], T_Result],
        us_gov_west1: typing.Callable[[], T_Result],
        us_west1: typing.Callable[[], T_Result],
        us_west2: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is AwsRegion.AF_SOUTH1:
            return af_south1()
        if self is AwsRegion.AP_EAST1:
            return ap_east1()
        if self is AwsRegion.AP_NORTHEAST1:
            return ap_northeast1()
        if self is AwsRegion.AP_NORTHEAST2:
            return ap_northeast2()
        if self is AwsRegion.AP_NORTHEAST3:
            return ap_northeast3()
        if self is AwsRegion.AP_SOUTH1:
            return ap_south1()
        if self is AwsRegion.AP_SOUTH2:
            return ap_south2()
        if self is AwsRegion.AP_SOUTHEAST1:
            return ap_southeast1()
        if self is AwsRegion.AP_SOUTHEAST2:
            return ap_southeast2()
        if self is AwsRegion.AP_SOUTHEAST3:
            return ap_southeast3()
        if self is AwsRegion.AP_SOUTHEAST4:
            return ap_southeast4()
        if self is AwsRegion.AP_SOUTHEAST5:
            return ap_southeast5()
        if self is AwsRegion.AP_SOUTHEAST7:
            return ap_southeast7()
        if self is AwsRegion.CA_CENTRAL1:
            return ca_central1()
        if self is AwsRegion.CA_WEST1:
            return ca_west1()
        if self is AwsRegion.CN_NORTH1:
            return cn_north1()
        if self is AwsRegion.CN_NORTHWEST1:
            return cn_northwest1()
        if self is AwsRegion.EU_CENTRAL1:
            return eu_central1()
        if self is AwsRegion.EU_CENTRAL2:
            return eu_central2()
        if self is AwsRegion.EU_NORTH1:
            return eu_north1()
        if self is AwsRegion.EU_SOUTH1:
            return eu_south1()
        if self is AwsRegion.EU_SOUTH2:
            return eu_south2()
        if self is AwsRegion.EU_WEST1:
            return eu_west1()
        if self is AwsRegion.EU_WEST2:
            return eu_west2()
        if self is AwsRegion.EU_WEST3:
            return eu_west3()
        if self is AwsRegion.IL_CENTRAL1:
            return il_central1()
        if self is AwsRegion.ME_CENTRAL1:
            return me_central1()
        if self is AwsRegion.ME_SOUTH1:
            return me_south1()
        if self is AwsRegion.MX_CENTRAL1:
            return mx_central1()
        if self is AwsRegion.SA_EAST1:
            return sa_east1()
        if self is AwsRegion.US_EAST1:
            return us_east1()
        if self is AwsRegion.US_EAST2:
            return us_east2()
        if self is AwsRegion.US_GOV_EAST1:
            return us_gov_east1()
        if self is AwsRegion.US_GOV_WEST1:
            return us_gov_west1()
        if self is AwsRegion.US_WEST1:
            return us_west1()
        if self is AwsRegion.US_WEST2:
            return us_west2()



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class ModelType(enum.StrEnum):
    """
    Model Type
    """

    CHAT = "chat"
    COMPLETION = "completion"
    EMBEDDING = "embedding"
    REALTIME = "realtime"
    RERANK = "rerank"
    AUDIO_TRANSCRIPTION = "audio_transcription"
    AUDIO_TRANSLATION = "audio_translation"
    TEXT_TO_SPEECH = "text_to_speech"
    MODERATION = "moderation"
    IMAGE = "image"

    def visit(
        self,
        chat: typing.Callable[[], T_Result],
        completion: typing.Callable[[], T_Result],
        embedding: typing.Callable[[], T_Result],
        realtime: typing.Callable[[], T_Result],
        rerank: typing.Callable[[], T_Result],
        audio_transcription: typing.Callable[[], T_Result],
        audio_translation: typing.Callable[[], T_Result],
        text_to_speech: typing.Callable[[], T_Result],
        moderation: typing.Callable[[], T_Result],
        image: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ModelType.CHAT:
            return chat()
        if self is ModelType.COMPLETION:
            return completion()
        if self is ModelType.EMBEDDING:
            return embedding()
        if self is ModelType.REALTIME:
            return realtime()
        if self is ModelType.RERANK:
            return rerank()
        if self is ModelType.AUDIO_TRANSCRIPTION:
            return audio_transcription()
        if self is ModelType.AUDIO_TRANSLATION:
            return audio_translation()
        if self is ModelType.TEXT_TO_SPEECH:
            return text_to_speech()
        if self is ModelType.MODERATION:
            return moderation()
        if self is ModelType.IMAGE:
            return image()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .custom_basic_auth import CustomBasicAuth


class CustomBlobStorage(UniversalBaseModel):
    """
    Custom Blob Storage
    """

    type: typing.Literal["integration/blob-storage/custom"] = pydantic.Field(default="integration/blob-storage/custom")
    """
    +value=integration/blob-storage/custom
    """

    name: str = pydantic.Field()
    """
    The name of the integration that will be displayed in the TrueFoundry UI.
    """

    endpoint: str = pydantic.Field()
    """
    The endpoint of the storage provider.
    """

    storage_root: str = pydantic.Field()
    """
    The S3-compatible storage path in the format 's3://<bucket-name>/<optional-path-prefix>'. Works with S3-compatible storage providers like MinIO, Ceph, and DigitalOcean Spaces. Example: 's3://mybucket' or 's3://mybucket/artifacts/production'.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    The region where the storage is located.
    """

    auth_data: CustomBasicAuth
    authorized_subjects: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of subjects that are authorized to access this integration. List of user fqn in format <user_type>:<username>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.alert_status import AlertStatus
from ..types.get_alerts_response import GetAlertsResponse
from .raw_client import AsyncRawAlertsClient, RawAlertsClient


class AlertsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAlertsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAlertsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAlertsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        start_ts: typing.Optional[str] = None,
        end_ts: typing.Optional[str] = None,
        cluster_id: typing.Optional[str] = None,
        application_id: typing.Optional[str] = None,
        alert_status: typing.Optional[AlertStatus] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetAlertsResponse:
        """
        Get alerts for a given application or cluster filtered by start and end timestamp

        Parameters
        ----------
        start_ts : typing.Optional[str]
            Start timestamp (ISO format) for querying events

        end_ts : typing.Optional[str]
            End timestamp (ISO format) for querying events

        cluster_id : typing.Optional[str]
            Cluster id

        application_id : typing.Optional[str]
            Application id

        alert_status : typing.Optional[AlertStatus]
            Alert status

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAlertsResponse
            Returns an object with alert name as key and list of alerts as value

        Examples
        --------
        from truefoundry_sdk import AlertStatus, TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.alerts.list(
            start_ts="startTs",
            end_ts="endTs",
            cluster_id="clusterId",
            application_id="applicationId",
            alert_status=AlertStatus.FIRING,
        )
        """
        _response = self._raw_client.list(
            start_ts=start_ts,
            end_ts=end_ts,
            cluster_id=cluster_id,
            application_id=application_id,
            alert_status=alert_status,
            request_options=request_options,
        )
        return _response.data


class AsyncAlertsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAlertsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAlertsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAlertsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        start_ts: typing.Optional[str] = None,
        end_ts: typing.Optional[str] = None,
        cluster_id: typing.Optional[str] = None,
        application_id: typing.Optional[str] = None,
        alert_status: typing.Optional[AlertStatus] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetAlertsResponse:
        """
        Get alerts for a given application or cluster filtered by start and end timestamp

        Parameters
        ----------
        start_ts : typing.Optional[str]
            Start timestamp (ISO format) for querying events

        end_ts : typing.Optional[str]
            End timestamp (ISO format) for querying events

        cluster_id : typing.Optional[str]
            Cluster id

        application_id : typing.Optional[str]
            Application id

        alert_status : typing.Optional[AlertStatus]
            Alert status

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAlertsResponse
            Returns an object with alert name as key and list of alerts as value

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AlertStatus, AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.alerts.list(
                start_ts="startTs",
                end_ts="endTs",
                cluster_id="clusterId",
                application_id="applicationId",
                alert_status=AlertStatus.FIRING,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            start_ts=start_ts,
            end_ts=end_ts,
            cluster_id=cluster_id,
            application_id=application_id,
            alert_status=alert_status,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .otel_exporter_grpc_config import OtelExporterGrpcConfig
from .otel_exporter_http_config import OtelExporterHttpConfig

GatewayOtelConfigOtelTracesExporterConfig = typing.Union[OtelExporterHttpConfig, OtelExporterGrpcConfig]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Permissions(UniversalBaseModel):
    resource_fqn: str = pydantic.Field()
    """
    The fully qualified name of the resource
    """

    resource_type: str = pydantic.Field()
    """
    The type of the resource (cluster, workspace, etc.)
    """

    role_id: str = pydantic.Field()
    """
    The role id of the role to be assigned to the service account for that resource
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .role_permissions import RolePermissions


class RoleManifest(UniversalBaseModel):
    type: typing.Literal["role"] = pydantic.Field(default="role")
    """
    +value=role
    """

    name: str = pydantic.Field()
    """
    Unique identifier of the role across the organization
    """

    display_name: typing_extensions.Annotated[str, FieldMetadata(alias="displayName")] = pydantic.Field()
    """
    Human-readable name for the role
    """

    description: str = pydantic.Field()
    """
    Description of the role that explains its purpose and permissions
    """

    resource_type: typing_extensions.Annotated[str, FieldMetadata(alias="resourceType")] = pydantic.Field()
    """
    Type of resource this role applies to
    """

    permissions: typing.List[RolePermissions] = pydantic.Field()
    """
    Define the resource type and the corresponding actions this role can perform on the resource
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.get_auto_provisioning_state_response import GetAutoProvisioningStateResponse
from .raw_client import AsyncRawClustersClient, RawClustersClient


class ClustersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawClustersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawClustersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawClustersClient
        """
        return self._raw_client

    def get_autoprovisioning_state(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetAutoProvisioningStateResponse:
        """
        Get the auto provisioning status for the provided cluster

        Parameters
        ----------
        id : str
            Cluster id of the cluster

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAutoProvisioningStateResponse
            Returns the auto provisioning status for the cluster

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.internal.clusters.get_autoprovisioning_state(
            id="id",
        )
        """
        _response = self._raw_client.get_autoprovisioning_state(id, request_options=request_options)
        return _response.data


class AsyncClustersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawClustersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawClustersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawClustersClient
        """
        return self._raw_client

    async def get_autoprovisioning_state(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetAutoProvisioningStateResponse:
        """
        Get the auto provisioning status for the provided cluster

        Parameters
        ----------
        id : str
            Cluster id of the cluster

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAutoProvisioningStateResponse
            Returns the auto provisioning status for the cluster

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.internal.clusters.get_autoprovisioning_state(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_autoprovisioning_state(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .artifacts_download import ArtifactsDownload
from .base_service_image import BaseServiceImage
from .base_service_mounts_item import BaseServiceMountsItem
from .health_probe import HealthProbe
from .kustomize import Kustomize
from .port import Port
from .resources import Resources


class BaseService(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    Name of the service. This uniquely identifies this service in the workspace.
    > Name can only contain alphanumeric characters and '-' and can be atmost 25 characters long
    """

    image: BaseServiceImage = pydantic.Field()
    """
    Specify whether you want to deploy a Docker image or build and deploy from source code
    """

    artifacts_download: typing.Optional[ArtifactsDownload] = None
    resources: typing.Optional[Resources] = None
    env: typing.Optional[typing.Dict[str, typing.Optional[str]]] = pydantic.Field(default=None)
    """
    Configure environment variables to be injected in the service either as plain text or secrets. [Docs](https://docs.truefoundry.com/docs/env-variables)
    """

    ports: typing.List[Port] = pydantic.Field()
    """
    Expose the deployment to make it accessible over the internet or keep it private. Implement authentication to restrict access. [Docs](https://docs.truefoundry.com/docs/define-ports-and-domains)
    """

    service_account: typing.Optional[str] = None
    mounts: typing.Optional[typing.List[BaseServiceMountsItem]] = pydantic.Field(default=None)
    """
    Configure data to be mounted to service pod(s) as a string, secret or volume. [Docs](https://docs.truefoundry.com/docs/mounting-volumes-service)
    """

    labels: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Labels
    """

    kustomize: typing.Optional[Kustomize] = None
    liveness_probe: typing.Optional[HealthProbe] = None
    readiness_probe: typing.Optional[HealthProbe] = None
    startup_probe: typing.Optional[HealthProbe] = None
    workspace_fqn: typing.Optional[str] = pydantic.Field(default=None)
    """
    Fully qualified name of the workspace
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .schedule_concurrency_policy import ScheduleConcurrencyPolicy


class Schedule(UniversalBaseModel):
    """
    Run the job on a schedule. [Docs](https://docs.truefoundry.com/docs/deploy-a-cron-job)
    """

    type: typing.Literal["scheduled"] = pydantic.Field(default="scheduled")
    """
    +value=scheduled
    """

    schedule: str = pydantic.Field()
    """
    Specify the schedule for this job to be run periodically in cron format.
    ```
    * * * * *
    | | | | |
    | | | | |___ day of week (0-6) (Sunday is 0)
    | | | |_____ month (1-12)
    | | |_______ day of month (1-31)
    | |_________ hour (0-23)
    |___________ minute (0-59)
    ```
    """

    concurrency_policy: ScheduleConcurrencyPolicy = pydantic.Field()
    """
    Choose whether to allow this job to run while another instance of the job is running, or to replace the currently running instance. Allow
    will enable multiple instances of this job to run. Forbid will keep the current instance of the job running and stop a new instance from being run.
    Replace will terminate any currently running instance of the job and start a new one.
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    Timezone against which the cron schedule will be calculated, e.g. "Asia/Tokyo". Default is machine's local time.
    https://docs.truefoundry.com/docs/list-of-supported-timezones
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .assistant_message_content_one_item import AssistantMessageContentOneItem
from .blob_storage_reference import BlobStorageReference

AssistantMessageContent = typing.Union[str, typing.List[AssistantMessageContentOneItem], BlobStorageReference]



# This file was auto-generated by Fern from our API Definition.

import typing

from .aws_access_key_based_auth import AwsAccessKeyBasedAuth
from .aws_assumed_role_based_auth import AwsAssumedRoleBasedAuth

AwsParameterStoreAuthData = typing.Union[AwsAccessKeyBasedAuth, AwsAssumedRoleBasedAuth]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .mcp_tool import McpTool


class McpServerWithUrl(UniversalBaseModel):
    """
    MCP server with URL
    """

    type: typing.Literal["mcp-server-url"] = pydantic.Field(default="mcp-server-url")
    """
    The URL of the MCP server
    """

    url: str = pydantic.Field()
    """
    The URL of the MCP server
    """

    headers: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    The headers to send to the MCP server
    """

    enable_all_tools: bool = pydantic.Field(default=False)
    """
    Whether to enable all tools from the MCP server
    """

    tools: typing.Optional[typing.List[McpTool]] = pydantic.Field(default=None)
    """
    The tools to enable from the MCP server
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .custom_blob_storage import CustomBlobStorage
from .custom_helm_repo import CustomHelmRepo
from .custom_jwt_auth_integration import CustomJwtAuthIntegration
from .custom_username_password_artifacts_registry import CustomUsernamePasswordArtifactsRegistry
from .email_notification_channel import EmailNotificationChannel

CustomIntegrations = typing.Union[
    CustomUsernamePasswordArtifactsRegistry,
    EmailNotificationChannel,
    CustomHelmRepo,
    CustomBlobStorage,
    CustomJwtAuthIntegration,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class MultiPartUploadStorageProvider(enum.StrEnum):
    """
    An enumeration.
    """

    S3COMPATIBLE = "S3_COMPATIBLE"
    AZURE_BLOB = "AZURE_BLOB"

    def visit(self, s3compatible: typing.Callable[[], T_Result], azure_blob: typing.Callable[[], T_Result]) -> T_Result:
        if self is MultiPartUploadStorageProvider.S3COMPATIBLE:
            return s3compatible()
        if self is MultiPartUploadStorageProvider.AZURE_BLOB:
            return azure_blob()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CanaryStep(UniversalBaseModel):
    weight_percentage: int = pydantic.Field()
    """
    Percentage of total traffic to be shifted to the canary release.
    The rest will continue to go to the existing deployment
    """

    pause_duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    Duration for which to pause the release. The release process will wait for these seconds before proceeding to the next step.
    If this is not set, the step will pause indefinitely on this step
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CommonToolsSettings(UniversalBaseModel):
    """
    Describes the configuration for Common Tools MCP Server in AI Gateway
    """

    type: typing.Literal["settings/common-tools"] = pydantic.Field(default="settings/common-tools")
    """
    +value=settings/common-tools
    """

    web_search: bool = pydantic.Field(default=True)
    """
    Enable web search tool
    Allows model to search the web for information.
    """

    code_executor: bool = pydantic.Field(default=True)
    """
    Enable code executor tool
    Allows model to execute code and return the results.
    """

    sequential_thinking: bool = pydantic.Field(default=True)
    """
    Enable sequential thinking tool
    Allows model to reason step-by-step to solve complex problems.
    """

    vision_ocr: bool = pydantic.Field(default=True)
    """
    Enable vision OCR tool
    Allows model to extract text from images or PDFs.
    """

    web_scrape: bool = pydantic.Field(default=True)
    """
    Enable web scraping tool.
    Allows model to scrape content from web pages with intelligent format selection for structured or unstructured data.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .mcp_server_auth import McpServerAuth
from .mcp_server_integration_transport import McpServerIntegrationTransport


class McpServerIntegration(UniversalBaseModel):
    """
    MCP Server
    """

    type: typing.Literal["integration/mcp-server/remote"] = pydantic.Field(default="integration/mcp-server/remote")
    """
    +value=integration/mcp-server/remote
    """

    name: str = pydantic.Field()
    """
    The name of the MCP Server.
    """

    description: str = pydantic.Field()
    """
    Provide a brief description of the purpose of this MCP Server.
    """

    url: str = pydantic.Field()
    """
    The endpoint URL for the MCP Server.
    """

    transport: McpServerIntegrationTransport = pydantic.Field()
    """
    The transport protocol to use for communication with the MCP Server.
    """

    auth_data: typing.Optional[McpServerAuth] = None
    authorized_subjects: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of teams/users who can use this MCP server.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .collaborator import Collaborator
from .mcp_server_integrations import McpServerIntegrations
from .owned_by import OwnedBy


class McpServerProviderAccount(UniversalBaseModel):
    """
    MCP Server Group
    """

    type: typing.Literal["provider-account/mcp-server-group"] = pydantic.Field(
        default="provider-account/mcp-server-group"
    )
    """
    +value=provider-account/mcp-server-group
    """

    name: str = pydantic.Field()
    """
    The name of the MCP Server Group.
    """

    collaborators: typing.List[Collaborator] = pydantic.Field()
    """
    List of teams/users who can manage this MCP Server Group. These subjects can add new MCP server integrations, can edit existing ones and can also use the MCP server.
    """

    integrations: typing.List[McpServerIntegrations] = pydantic.Field()
    """
    List of MCP Servers, which are part of this MCP Server Group.
    """

    owned_by: typing_extensions.Annotated[typing.Optional[OwnedBy], FieldMetadata(alias="ownedBy")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



from typing import Any, Optional, Protocol, TypeVar

from .applications.client import ApplicationsClient, AsyncApplicationsClient
from .artifact_versions.client import ArtifactVersionsClient, AsyncArtifactVersionsClient
from .artifacts.client import ArtifactsClient, AsyncArtifactsClient
from .core.pagination import AsyncPager, SyncPager
from .core.pydantic_utilities import parse_obj_as
from .core.request_options import RequestOptions
from .data_directories.client import AsyncDataDirectoriesClient, DataDirectoriesClient
from .errors import NotFoundError
from .model_versions.client import AsyncModelVersionsClient, ModelVersionsClient
from .models.client import AsyncModelsClient, ModelsClient
from .prompt_versions.client import AsyncPromptVersionsClient, PromptVersionsClient
from .prompts.client import AsyncPromptsClient, PromptsClient
from .secret_groups.client import AsyncSecretGroupsClient, SecretGroupsClient
from .types.get_application_response import GetApplicationResponse
from .types.get_artifact_response import GetArtifactResponse
from .types.get_artifact_version_response import GetArtifactVersionResponse
from .types.get_data_directory_response import GetDataDirectoryResponse
from .types.get_model_response import GetModelResponse
from .types.get_model_version_response import GetModelVersionResponse
from .types.get_prompt_response import GetPromptResponse
from .types.get_prompt_version_response import GetPromptVersionResponse
from .types.get_secret_group_response import GetSecretGroupResponse
from .types.get_workspace_response import GetWorkspaceResponse
from .types.http_error import HttpError
from .workspaces.client import AsyncWorkspacesClient, WorkspacesClient
from pydantic import BaseModel

T = TypeVar("T", bound=BaseModel)
R = TypeVar("R", bound=BaseModel)


class HasListMethod(Protocol[T, R]):
    def list(self, *, fqn: str, limit: Optional[int] = None, **kwargs: Any) -> SyncPager[T, R]: ...


class HasAsyncListMethod(Protocol[T, R]):
    async def list(self, *, fqn: str, limit: Optional[int] = None, **kwargs) -> AsyncPager[T, R]: ...


def _get_by_fqn(client: HasListMethod[T, R], *, fqn: str, request_options: Optional[RequestOptions] = None) -> T:
    result = None
    for item in client.list(fqn=fqn, limit=1, request_options=request_options):
        result = item
        break
    if not result:
        raise NotFoundError(
            body=HttpError(
                message=f"No entity found with fqn {fqn}",
                status_code=404,
            )
        )
    return result


async def _aget_by_fqn(
    client: HasAsyncListMethod[T, R], *, fqn: str, request_options: Optional[RequestOptions] = None
) -> T:
    result = None
    pager = await client.list(fqn=fqn, limit=1, request_options=request_options)
    async for item in pager:
        result = item
        break

    if not result:
        raise NotFoundError(
            body=HttpError(
                message=f"No entity found with fqn {fqn}",
                status_code=404,
            )
        )
    return result


class WrappedApplicationsClient(ApplicationsClient):
    def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetApplicationResponse:
        """
        Get Application by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the application

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetApplicationResponse
            Application details

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.applications.get_by_fqn(
            fqn="fqn",
        )
        """
        item = _get_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetApplicationResponse, {"data": item})


class WrappedArtifactsClient(ArtifactsClient):
    def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetArtifactResponse:
        """
        Get Artifact by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the artifact

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetArtifactResponse
            Artifact details

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.artifacts.get_by_fqn(
            fqn="fqn",
        )
        """
        item = _get_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetArtifactResponse, {"data": item})


class WrappedArtifactVersionsClient(ArtifactVersionsClient):
    def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetArtifactVersionResponse:
        """
        Get Artifact Version by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the artifact version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetArtifactVersionResponse
            Artifact version details

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.artifact_versions.get_by_fqn(
            fqn="fqn",
        )
        """
        item = _get_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetArtifactVersionResponse, {"data": item})


class WrappedDataDirectoriesClient(DataDirectoriesClient):
    def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetDataDirectoryResponse:
        """
        Get Data Directory by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the data directory

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDataDirectoryResponse
            Data directory details

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.data_directories.get_by_fqn(
            fqn="fqn",
        )
        """
        item = _get_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetDataDirectoryResponse, {"data": item})


class WrappedModelsClient(ModelsClient):
    def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetModelResponse:
        """
        Get Model by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the model

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetModelResponse
            Model details

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.models.get_by_fqn(
            fqn="fqn",
        )
        """
        item = _get_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetModelResponse, {"data": item})


class WrappedModelVersionsClient(ModelVersionsClient):
    def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetModelVersionResponse:
        """
        Get Model Version by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the model version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetModelVersionResponse
            Model version details

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.model_versions.get_by_fqn(
            fqn="fqn",
        )
        """
        item = _get_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetModelVersionResponse, {"data": item})


class WrappedPromptsClient(PromptsClient):
    def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetPromptResponse:
        """
        Get Prompt by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the prompt

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPromptResponse
            Prompt details

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.prompts.get_by_fqn(
            fqn="fqn",
        )
        """
        item = _get_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetPromptResponse, {"data": item})


class WrappedPromptVersionsClient(PromptVersionsClient):
    def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetPromptVersionResponse:
        """
        Get Prompt Version by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the prompt version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPromptVersionResponse
            Prompt version details

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.prompt_versions.get_by_fqn(
            fqn="fqn",
        )
        """
        item = _get_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetPromptVersionResponse, {"data": item})


class WrappedSecretGroupsClient(SecretGroupsClient):
    def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetSecretGroupResponse:
        """
        Get Secret Group by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the secret group

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSecretGroupResponse
            Secret group details

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.secret_groups.get_by_fqn(
            fqn="fqn",
        )
        """
        item = _get_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetSecretGroupResponse, {"data": item})


class WrappedWorkspacesClient(WorkspacesClient):
    def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetWorkspaceResponse:
        """
        Get Workspace by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the workspace

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetWorkspaceResponse
            Workspace details

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.workspaces.get_by_fqn(
            fqn="fqn",
        )
        """
        item = _get_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetWorkspaceResponse, {"data": item})


class WrappedAsyncApplicationsClient(AsyncApplicationsClient):
    async def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetApplicationResponse:
        """
        Get Application by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the application

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetApplicationResponse
            Application details

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )

        async def main() -> None:
            await client.applications.get_by_fqn(
                fqn="fqn",
            )

        asyncio.run(main())
        """
        item = await _aget_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetApplicationResponse, {"data": item})


class WrappedAsyncArtifactsClient(AsyncArtifactsClient):
    async def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetArtifactResponse:
        """
        Get Artifact by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the artifact

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetArtifactResponse
            Artifact details

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )

        async def main() -> None:
            await client.artifacts.get_by_fqn(
                fqn="fqn",
            )

        asyncio.run(main())
        """
        item = await _aget_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetArtifactResponse, {"data": item})


class WrappedAsyncArtifactVersionsClient(AsyncArtifactVersionsClient):
    async def get_by_fqn(
        self, fqn: str, *, request_options: Optional[RequestOptions] = None
    ) -> GetArtifactVersionResponse:
        """
        Get Artifact Version by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the artifact version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetArtifactVersionResponse
            Artifact version details

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )

        async def main() -> None:
            await client.artifact_versions.get_by_fqn(
                fqn="fqn",
            )

        asyncio.run(main())
        """
        item = await _aget_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetArtifactVersionResponse, {"data": item})


class WrappedAsyncDataDirectoriesClient(AsyncDataDirectoriesClient):
    async def get_by_fqn(
        self, fqn: str, *, request_options: Optional[RequestOptions] = None
    ) -> GetDataDirectoryResponse:
        """
        Get Data Directory by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the data directory

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDataDirectoryResponse
            Data directory details

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )

        async def main() -> None:
            await client.data_directories.get_by_fqn(
                fqn="fqn",
            )

        asyncio.run(main())
        """
        item = await _aget_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetDataDirectoryResponse, {"data": item})


class WrappedAsyncModelsClient(AsyncModelsClient):
    async def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetModelResponse:
        """
        Get Model by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the model

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetModelResponse
            Model details

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )

        async def main() -> None:
            await client.models.get_by_fqn(
                fqn="fqn",
            )

        asyncio.run(main())
        """
        item = await _aget_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetModelResponse, {"data": item})


class WrappedAsyncModelVersionsClient(AsyncModelVersionsClient):
    async def get_by_fqn(
        self, fqn: str, *, request_options: Optional[RequestOptions] = None
    ) -> GetModelVersionResponse:
        """
        Get Model Version by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the model version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetModelVersionResponse
            Model version details

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )

        async def main() -> None:
            await client.model_versions.get_by_fqn(
                fqn="fqn",
            )

        asyncio.run(main())
        """
        item = await _aget_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetModelVersionResponse, {"data": item})


class WrappedAsyncPromptsClient(AsyncPromptsClient):
    async def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetPromptResponse:
        """
        Get Prompt by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the prompt

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPromptResponse
            Prompt details

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )

        async def main() -> None:
            await client.prompts.get_by_fqn(
                fqn="fqn",
            )

        asyncio.run(main())
        """
        item = await _aget_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetPromptResponse, {"data": item})


class WrappedAsyncPromptVersionsClient(AsyncPromptVersionsClient):
    async def get_by_fqn(
        self, fqn: str, *, request_options: Optional[RequestOptions] = None
    ) -> GetPromptVersionResponse:
        """
        Get Prompt Version by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the prompt version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPromptVersionResponse
            Prompt version details

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )

        async def main() -> None:
            await client.prompt_versions.get_by_fqn(
                fqn="fqn",
            )

        asyncio.run(main())
        """
        item = await _aget_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetPromptVersionResponse, {"data": item})


class WrappedAsyncSecretGroupsClient(AsyncSecretGroupsClient):
    async def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetSecretGroupResponse:
        """
        Get Secret Group by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the secret group

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSecretGroupResponse
            Secret group details

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )

        async def main() -> None:
            await client.secret_groups.get_by_fqn(
                fqn="fqn",
            )

        asyncio.run(main())
        """
        item = await _aget_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetSecretGroupResponse, {"data": item})


class WrappedAsyncWorkspacesClient(AsyncWorkspacesClient):
    async def get_by_fqn(self, fqn: str, *, request_options: Optional[RequestOptions] = None) -> GetWorkspaceResponse:
        """
        Get Workspace by FQN.

        Parameters
        ----------
        fqn : str
            FQN of the workspace

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetWorkspaceResponse
            Workspace details

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )

        async def main() -> None:
            await client.workspaces.get_by_fqn(
                fqn="fqn",
            )

        asyncio.run(main())
        """
        item = await _aget_by_fqn(self, fqn=fqn, request_options=request_options)  # type: ignore[arg-type,var-annotated]
        return parse_obj_as(GetWorkspaceResponse, {"data": item})



# This file was auto-generated by Fern from our API Definition.

import typing

from .aws_provider_account import AwsProviderAccount
from .azure_provider_account import AzureProviderAccount
from .bitbucket_provider_account import BitbucketProviderAccount
from .custom_provider_account import CustomProviderAccount
from .dockerhub_provider_account import DockerhubProviderAccount
from .gcp_provider_account import GcpProviderAccount
from .github_provider_account import GithubProviderAccount
from .gitlab_provider_account import GitlabProviderAccount
from .hashicorp_provider_account import HashicorpProviderAccount
from .jfrog_provider_account import JfrogProviderAccount
from .pager_duty_provider_account import PagerDutyProviderAccount
from .quay_provider_account import QuayProviderAccount
from .slack_provider_account import SlackProviderAccount
from .true_foundry_provider_account import TrueFoundryProviderAccount
from .ttl_provider_account import TtlProviderAccount
from .webhook_provider_account import WebhookProviderAccount

InfraProviderAccount = typing.Union[
    AwsProviderAccount,
    AzureProviderAccount,
    GcpProviderAccount,
    DockerhubProviderAccount,
    BitbucketProviderAccount,
    CustomProviderAccount,
    GithubProviderAccount,
    GitlabProviderAccount,
    JfrogProviderAccount,
    TtlProviderAccount,
    TrueFoundryProviderAccount,
    QuayProviderAccount,
    SlackProviderAccount,
    WebhookProviderAccount,
    PagerDutyProviderAccount,
    HashicorpProviderAccount,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.environment import Environment
from ..types.environment_manifest import EnvironmentManifest
from ..types.get_environment_response import GetEnvironmentResponse
from ..types.list_environments_response import ListEnvironmentsResponse
from .raw_client import AsyncRawEnvironmentsClient, RawEnvironmentsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EnvironmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEnvironmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEnvironmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEnvironmentsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Environment, ListEnvironmentsResponse]:
        """
        List environments, if no environments are found, default environments are created and returned. Pagination is available based on query parameters

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of items per page

        offset : typing.Optional[int]
            Number of items to skip

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Environment, ListEnvironmentsResponse]
            Returns a list of environment. If pagination parameters are provided, the response includes paginated data

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        response = client.environments.list(
            limit=10,
            offset=0,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(limit=limit, offset=offset, request_options=request_options)

    def create_or_update(
        self,
        *,
        manifest: EnvironmentManifest,
        dry_run: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEnvironmentResponse:
        """
        Creates a new Environment or updates an existing Environment.

        Parameters
        ----------
        manifest : EnvironmentManifest
            Environment Manifest

        dry_run : typing.Optional[bool]
            Dry run

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEnvironmentResponse
            Returns the created or updated Environment

        Examples
        --------
        from truefoundry_sdk import (
            EnvironmentColor,
            EnvironmentManifest,
            EnvironmentOptimizeFor,
            TrueFoundry,
        )

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.environments.create_or_update(
            manifest=EnvironmentManifest(
                name="name",
                color=EnvironmentColor(),
                is_production=True,
                optimize_for=EnvironmentOptimizeFor.COST,
            ),
        )
        """
        _response = self._raw_client.create_or_update(
            manifest=manifest, dry_run=dry_run, request_options=request_options
        )
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetEnvironmentResponse:
        """
        Get Environment associated with the provided id.

        Parameters
        ----------
        id : str
            Environment id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEnvironmentResponse
            Returns the Environment associated with the provided id

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.environments.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> bool:
        """
        Delete Environment associated with the provided id.

        Parameters
        ----------
        id : str
            Environment id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        bool
            Returns true if the Environment is deleted successfully

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.environments.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data


class AsyncEnvironmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEnvironmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEnvironmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEnvironmentsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Environment, ListEnvironmentsResponse]:
        """
        List environments, if no environments are found, default environments are created and returned. Pagination is available based on query parameters

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of items per page

        offset : typing.Optional[int]
            Number of items to skip

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Environment, ListEnvironmentsResponse]
            Returns a list of environment. If pagination parameters are provided, the response includes paginated data

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            response = await client.environments.list(
                limit=10,
                offset=0,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(limit=limit, offset=offset, request_options=request_options)

    async def create_or_update(
        self,
        *,
        manifest: EnvironmentManifest,
        dry_run: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEnvironmentResponse:
        """
        Creates a new Environment or updates an existing Environment.

        Parameters
        ----------
        manifest : EnvironmentManifest
            Environment Manifest

        dry_run : typing.Optional[bool]
            Dry run

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEnvironmentResponse
            Returns the created or updated Environment

        Examples
        --------
        import asyncio

        from truefoundry_sdk import (
            AsyncTrueFoundry,
            EnvironmentColor,
            EnvironmentManifest,
            EnvironmentOptimizeFor,
        )

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.environments.create_or_update(
                manifest=EnvironmentManifest(
                    name="name",
                    color=EnvironmentColor(),
                    is_production=True,
                    optimize_for=EnvironmentOptimizeFor.COST,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_or_update(
            manifest=manifest, dry_run=dry_run, request_options=request_options
        )
        return _response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetEnvironmentResponse:
        """
        Get Environment associated with the provided id.

        Parameters
        ----------
        id : str
            Environment id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEnvironmentResponse
            Returns the Environment associated with the provided id

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.environments.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> bool:
        """
        Delete Environment associated with the provided id.

        Parameters
        ----------
        id : str
            Environment id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        bool
            Returns true if the Environment is deleted successfully

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.environments.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.not_found_error import NotFoundError
from ...types.application_type import ApplicationType
from ...types.deployment_build import DeploymentBuild
from ...types.deployment_status import DeploymentStatus
from ...types.get_suggested_deployment_endpoint_response import GetSuggestedDeploymentEndpointResponse
from ...types.presigned_url_object import PresignedUrlObject

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDeploymentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_deployment_statuses(
        self, id: str, deployment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[DeploymentStatus]]:
        """
        This endpoint returns all statuses for a specific deployment in a given application.

        Parameters
        ----------
        id : str
            Application id of the application

        deployment_id : str
            Deployment id of the deployment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[DeploymentStatus]]
            Deployment statuses returned successfully.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/svc/v1/apps/{jsonable_encoder(id)}/deployments/{jsonable_encoder(deployment_id)}/statuses",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[DeploymentStatus],
                    parse_obj_as(
                        type_=typing.List[DeploymentStatus],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_builds(
        self, id: str, deployment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[DeploymentBuild]]:
        """
        This endpoint returns all build details associated with a specific deployment in a given application.

        Parameters
        ----------
        id : str
            Application id of the application

        deployment_id : str
            Deployment id of the deployment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[DeploymentBuild]]
            Deployment builds returned successfully.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/svc/v1/apps/{jsonable_encoder(id)}/deployments/{jsonable_encoder(deployment_id)}/builds",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[DeploymentBuild],
                    parse_obj_as(
                        type_=typing.List[DeploymentBuild],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_code_upload_url(
        self, *, service_name: str, workspace_fqn: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PresignedUrlObject]:
        """
        Generate presigned URL to upload code for given serviceName and workspaceFqn

        Parameters
        ----------
        service_name : str
            ServiceName of the deployment

        workspace_fqn : str
            WorkSpaceFQN of the workspace

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PresignedUrlObject]
            Returns a presigned URL and URI as json object
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/svc/v1/deployment/code-upload-url",
            method="POST",
            json={
                "serviceName": service_name,
                "workspaceFqn": workspace_fqn,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PresignedUrlObject,
                    parse_obj_as(
                        type_=PresignedUrlObject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_suggested_endpoint(
        self,
        *,
        application_type: ApplicationType,
        application_name: str,
        workspace_id: str,
        base_domain: typing.Optional[str] = None,
        port: typing.Optional[str] = None,
        prefer_wildcard: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetSuggestedDeploymentEndpointResponse]:
        """
        Generate deployment endpoint based on the provided query parameters.

        Parameters
        ----------
        application_type : ApplicationType
            Application Type

        application_name : str
            Application name

        workspace_id : str
            Workspace id

        base_domain : typing.Optional[str]
            Base domain

        port : typing.Optional[str]
            Port

        prefer_wildcard : typing.Optional[bool]
            Prefer wildcard

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSuggestedDeploymentEndpointResponse]
            Returns host and path as json object
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/svc/v1/deployment/deployment-endpoint",
            method="GET",
            params={
                "applicationType": application_type,
                "applicationName": application_name,
                "workspaceId": workspace_id,
                "baseDomain": base_domain,
                "port": port,
                "preferWildcard": prefer_wildcard,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSuggestedDeploymentEndpointResponse,
                    parse_obj_as(
                        type_=GetSuggestedDeploymentEndpointResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDeploymentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_deployment_statuses(
        self, id: str, deployment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[DeploymentStatus]]:
        """
        This endpoint returns all statuses for a specific deployment in a given application.

        Parameters
        ----------
        id : str
            Application id of the application

        deployment_id : str
            Deployment id of the deployment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[DeploymentStatus]]
            Deployment statuses returned successfully.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/svc/v1/apps/{jsonable_encoder(id)}/deployments/{jsonable_encoder(deployment_id)}/statuses",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[DeploymentStatus],
                    parse_obj_as(
                        type_=typing.List[DeploymentStatus],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_builds(
        self, id: str, deployment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[DeploymentBuild]]:
        """
        This endpoint returns all build details associated with a specific deployment in a given application.

        Parameters
        ----------
        id : str
            Application id of the application

        deployment_id : str
            Deployment id of the deployment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[DeploymentBuild]]
            Deployment builds returned successfully.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/svc/v1/apps/{jsonable_encoder(id)}/deployments/{jsonable_encoder(deployment_id)}/builds",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[DeploymentBuild],
                    parse_obj_as(
                        type_=typing.List[DeploymentBuild],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_code_upload_url(
        self, *, service_name: str, workspace_fqn: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PresignedUrlObject]:
        """
        Generate presigned URL to upload code for given serviceName and workspaceFqn

        Parameters
        ----------
        service_name : str
            ServiceName of the deployment

        workspace_fqn : str
            WorkSpaceFQN of the workspace

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PresignedUrlObject]
            Returns a presigned URL and URI as json object
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/svc/v1/deployment/code-upload-url",
            method="POST",
            json={
                "serviceName": service_name,
                "workspaceFqn": workspace_fqn,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PresignedUrlObject,
                    parse_obj_as(
                        type_=PresignedUrlObject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_suggested_endpoint(
        self,
        *,
        application_type: ApplicationType,
        application_name: str,
        workspace_id: str,
        base_domain: typing.Optional[str] = None,
        port: typing.Optional[str] = None,
        prefer_wildcard: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetSuggestedDeploymentEndpointResponse]:
        """
        Generate deployment endpoint based on the provided query parameters.

        Parameters
        ----------
        application_type : ApplicationType
            Application Type

        application_name : str
            Application name

        workspace_id : str
            Workspace id

        base_domain : typing.Optional[str]
            Base domain

        port : typing.Optional[str]
            Port

        prefer_wildcard : typing.Optional[bool]
            Prefer wildcard

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSuggestedDeploymentEndpointResponse]
            Returns host and path as json object
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/svc/v1/deployment/deployment-endpoint",
            method="GET",
            params={
                "applicationType": application_type,
                "applicationName": application_name,
                "workspaceId": workspace_id,
                "baseDomain": base_domain,
                "port": port,
                "preferWildcard": prefer_wildcard,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSuggestedDeploymentEndpointResponse,
                    parse_obj_as(
                        type_=GetSuggestedDeploymentEndpointResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .assistant_message_content import AssistantMessageContent
from .tool_call import ToolCall


class AssistantMessage(UniversalBaseModel):
    """
    Assistant message turn
    """

    role: typing.Literal["assistant"] = pydantic.Field(default="assistant")
    """
    Role of the message
    """

    content: typing.Optional[AssistantMessageContent] = pydantic.Field(default=None)
    """
    Text content for the assistant message
    """

    tool_calls: typing.Optional[typing.List[ToolCall]] = pydantic.Field(default=None)
    """
    List of tool calls made by the assistant
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the assistant this message is from
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .budget_alert import BudgetAlert
from .budget_limit_unit import BudgetLimitUnit
from .budget_when import BudgetWhen


class BudgetRule(UniversalBaseModel):
    """
    Budget Rule
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the rule
    """

    when: BudgetWhen
    limit_to: float = pydantic.Field()
    """
    Numeric limit value for the budget
    """

    unit: BudgetLimitUnit
    budget_applies_per: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Create separate budget rules for each unique value of the selected entity. For example, if "user" is selected, a separate budget rule will be created for each unique user making requests. Options: user, virtualaccount, model, or a metadata key (e.g., metadata.appId).
    """

    alerts: typing.Optional[BudgetAlert] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow
